%% Script to calculate loop filter parameters for Cppsim schematic
% Ideal loop filter without excess loop delay
clear all;

% Be sure to add the path of the Delta Sigma Toolbox installation
% addpath('C:/.../delsig');

% Setup parameters
Fs = 0.95e9;
BW_analog = 20e6;
R = 25;

% Synthesize ideal NTF given input parameters
H = synthesizeNTF(4,R,1); % Fourth order optimized for 20MHz
A = 1/H-1; % Calculate ideal open loop TF for chosen NTF

% Plot ideal NTF 
f = [linspace(0,0.75/R,100) linspace(0.75/R,1,500)];
s = j*2*pi*f;
z = exp(s);
z_d = filt([0 1],1,1);

figure(1); clf;
subplot(321);
plot(f,dbv(evalTF(H,z)),'k','Linewidth',2);
axis([0 0.5 -90 40]);
hold on;
grid on;
title('Desired NTF');

% Plot ideal STF in black 
subplot(323);
plot(f,dbv(evalTF(A/(1+A),z)),'k','Linewidth',2);
axis([0 0.5 -20 40]);
hold on;
grid on;
title('Desired STF');

% Plot ideal impulse response in black 
subplot(325);
imp_l = 20;
y = impulse(A/(1+A),1:imp_l);
%stem(1:imp_l,y,'k','Linewidth',2);
stem(1:imp_l,y,'k')
hold on;
axis([0 imp_l -2 2]);
title('Desired Impulse Response');

%% Add excess loop delay
% Now include the excess loop delay, vary the gain of the minor 
% loop DAC to compensate for the delay, and look at the effects 
% on the STF and the impulse response
k = 0:0.25:1;
k_l = length(k);
cmap = colormap(jet(k_l));
for i = 1:k_l,
    % Pick a DAC gain
    K = filt(k(i),1,1);
    
    % Calculate the new discrete-time open loop TF "Ad" that has the same 
    % NTF as the ideal "A"
    Ad = A*tf([1 0],1,1)-K; 

    % Plot the effect on the NTF
    subplot(321);
    plot(f,dbv(evalTF(1/(1+(Ad+K)*z_d),z)),'Color',cmap(i,:));
    hold on;
    grid on;
    title('NTF for varied DAC gains');
    
    % Plot the effect on the STF
    subplot(323);
    plot(f,dbv(evalTF(Ad/(1+(Ad+K)*z_d),z)),'Color',cmap(i,:));
    hold on;
    grid on;
    title('STF for varied DAC gains');

    % and plot the effect on the impulse response
    subplot(325);
    y = impulse(Ad/(1+(Ad+K)*z_d),1:imp_l);
    %stem(1:imp_l,y,'Color',cmap(i,:));
    stem(1:imp_l,y,'k')
    hold on;
    title('Impulse response for varied DAC gains');
    
end

%% Based on these results, calculate the final CT open loop TF
k = 0.5;
K = filt(k,1,1);
Ad = A*tf([1 0],1,1)-K; % for making NTF equal to ideal
Ad.Ts = 1/Fs;
Ac = d2c(Ad,'tustin');

% Plot the open and closed loop magnitude response
f = Fs*[linspace(0,0.75/R,100) linspace(0.75/R,1,500)];
subplot(222);
semilogx(f,dbv(squeeze(freqresp(Ac,2*pi*f))),'r','LineWidth',1.5);
grid on;
hold on;
axis([1e6 1e9 -20 100]);
title('Final Open-Loop Magnitude Response');

% Plot the open and closed loop phase response
subplot(224);
semilogx(f,angle(squeeze(freqresp(Ac,2*pi*f))),'r','LineWidth',1.5);
grid on;
hold on;
axis([1e6 1e9 -4 4]);
title('Final Open-Loop Phase Response');

%% Use the other loop parameters to design the optimal 4th order loop filter
Kv = 7.35e8;                % Hz/V
Avco = 2*pi*Kv/Fs;          % rad/sample/V
Adac1 = 640e-6*31/pi;       % A/rad/sample
Adac2 = k/Avco;             % V/rad/sample
Alf_optimal = Ac/Avco/Adac1;

Alf_optimal = tf(Alf_optimal)
